# Exerciții

> Scheletul laboratorului se regaseste la adresa de [aici](https://gitlab.cs.pub.ro/pcom/pcom-laboratoare-public/-/tree/master/lab11). Funcționalitatea disponibilă este descrisă în README.md. În continuare, vom folosi Mininet pentru a simula o topologie formată din 3 host-uri conectate la un router: sudo python3 topo.py.

Pentru a observa anumite probleme și soluții legate de securitate, vom avea în
vedere următorul scenariu:

- Bob oferă un serviciu de capitalizare a datelor; un client se poate conecta la
  severul său, să îi trimită pe rând șiruri de date și să primească varianta
  capitalizată a inputului. (`server.c`)
- Alice vrea să folosească serviciul lui Bob. (`client.c`)
- Comunicarea dintre cei doi este intermediată de un router compromis, sub
  controlul unui atacator. (`attacker.c`)

Găsiți în scheletul de laborator o topologie `topo.py` care descrie o topologie
cu un singur router compromis, situat între Alice și Bob.

Rulați `make` și rulați topologia (`sudo python3 topo.py`):

 - rulați `./attacker` în terminalul corespunzător routerului (numit "r")
   (rulați `./attacker 2>/dev/null` pentru a elimina outputul de logging)
 - rulați `make run_server` pe terminalul `bob`.
 - rulați `make run_client` pe terminalul `alice`.
 - din terminalul lui Alice, scrieți mesaje din linia de comandă; acestea vor
   circula către server, prin routerul compromis; serverul le va prelucra și va
   transmite înapoi răspunsul.
 - observați cum atacatorul poate vedea tot ce comunică Alice și Bob

1\. **Comunicare criptată.** Pentru ca atacatorul să nu mai poată vedea ce
comunică Alice și Bob între ei, modificați atât clientul cât și serverul astfel
încât toate datele să fie criptate.
Pentru ca Alice și Bob să aibă aceeași cheie, aceasta va fi generată de Alice și
transmisă în primul mesaj către Bob. Vim folosi urmatorul API din `tea.h`. Vom
face lucra in `client.c` si `server.c`.

```c
/* Cripteaza un mesaj cu cheia k */
uint8_t *encrypt(uint8_t *plaintext, uint32_t *size, const uint32_t *k);
/* Decripteaza un mesaj cu cheia k */
uint8_t *decrypt(uint8_t *cipher, uint32_t *size, const uint32_t *k);

/* Creeaza o cheie secreta k. Dimensiunea cheii este sizeof(uint32_t) * 4 */
uint32_t *create_key();
```

2\. **Interceptare cheie.** La exercițiul anterior am criptat toate mesajele
dintre Alice și Bob; dar cheia de criptare a fost transmisă prin același canal de
comunicare compromis. Modificați codul atacatorului astfel încât să intercepteze
cheia transmisă și apoi să decripteze fiecare mesaj. Pentru acest exercitiu
vom lucra in `attacker.c`.

3\. **Diffie-Hellmann key-exchange.** Implementați mecanismul Diffie-Hellmann de
schimbare de cheie peste un canal nesecurizat. Astfel Alice și Bob vor ajunge să
aibă aceeași cheie de criptare, fără a fi nevoiți să o transfere explicit peste
canalul compromis. Vom folosi urmatorul API din `dh.h`.

```c
/* Calculeaza g^a mod p */
uint32_t mod_pow(uint32_t g, uint32_t a, uint32_t p);
/* Generates un secret, secret DH, pe baza numaruilui prim public p. 
   Acesta este valoarea a in cazul lui Alice si b in cazul lui Bob.*/
uint32_t generate_secret(uint32_t p);

/* Takes a shared secret and generates an encryption key. The difference between
 * the two is as follow. Generate_secret returns a for alice. They
 * compute g^(ab) mod p. This is the shared secret. But you now want to
 * have a key that we derive from this secret. We need to do this because
 * the shared secret may have different structure or size. */
uint32_t *derive_key(uint32_t shared_secret);
```

4\. **Man-in-the-Middle (MitM).** Deși atacatorul nu poate determina cheia schimbată cu
DH, el poate altera mesajele din tranzit, încât să transmită valori derivate din
propriul secret. Atfel, el poate să facă două instanțe de DH: una cu Alice și
una cu Bob; în urma cărora va împărți cu fiecare câte un secret.
Astfel, atacatorul poate decripta mesajele de la Alice, le poate citi, și apoi
cripta la loc pentru a i le trimite lui Bob; compromițând astfel comunicarea
dintre cei doi fără a fi detectat.
Modificați atacatorul astfel încât să modifice mesajele în tranzit și să ajungă
să facă DH cu Alice și Bob; apoi folosiți cheile obținute pentru a decripta
mesajele transmise. Pentru acest exercitiu vom lucra in `attacker.c`.

Pentru a a preveni MitM în internet, e nevoie ca valorile publice interschimbate
să fie însoțite de un certificat semnat de o entitate de încredere.
